/*   plotX.c - all the X10 routines  ***/

#include <X/Xlib.h>
#include <X/Xkeyboard.h>
#include <stdio.h>
#include <strings.h>
#include <math.h>
#include "header.h" 
#include "Xdefs.h"

#include "icon.ic"
#include "iconMask.ic"

#define LEFT  0
#define RIGHT 1
#define BACK  2
#define FRONT 3

#define X_DIM            600
#define Y_DIM            500
#define BDR_DIM          100
#define X_MKR            5
#define X_LIN            11
#define VIEW_X_ORIGIN    0
#define VIEW_Y_ORIGIN    0
#define VIEW_X_DIM       1000 
#define VIEW_Y_DIM       1000 
#define NLINES           2
#define DEFAULT_BORDER_WIDTH 3
#define DEFAULT_FONT         "helv12b"
#define DEFAULT_POSITION     "=%dx%d+0+0"   /* upper left hand corner */
#define DEFAULT_BORDER_COLOR "Blue"
#define DEFAULT_BACK_COLOR   "DarkBlue"
#define DEFAULT_FORE_COLOR   "White"
#define DEFAULT_MOUSE_COLOR  "White"
#define DEFAULT_ICON_COLOR   "Red"

#define Long_Dotted XMakePattern(0x0101,16,1)  /* Line type */
#define SolidSpace  XMakePattern(0x0000,16,1)  /* Line type */
#define Double_Dot  XMakePattern(0x0660,16,1)  /* Line type */
#define Long_Dash   XMakePattern(0x3f3f,16,1)  /* Line type */
#define Dot_Dash    XMakePattern(0x87e1,16,1)  /* Line type */
#define Long_Dots   XMakePattern(0x6666,16,1)  /* Line type */
#define Short_Dash  XMakePattern(0x0707,16,1)  /* Line type */
#define Equal_Dash  XMakePattern(0x0f0f,16,1)  /* Line type */
#define DotDot_Dash XMakePattern(0x4ff2,16,1)  /* Line type */

#define KC_A    0302                    /* interesting letter keycodes */
#define KC_B    0331   
#define KC_C    0316 
#define KC_D    0315
#define KC_E    0314
#define KC_F    0322
#define KC_G    0330
#define KC_H    0335
#define KC_I    0346
#define KC_J    0342
#define KC_K    0347
#define KC_L    0354
#define KC_M    0343
#define KC_N    0336
#define KC_O    0353
#define KC_P    0360
#define KC_Q    0301
#define KC_R    0321
#define KC_S    0307
#define KC_T    0327
#define KC_U    0341
#define KC_V    0323
#define KC_W    0306
#define KC_X    0310
#define KC_Y    0334
#define KC_Z    0303
#define KC_DEL  0274
#define KC_BAR  0324

/* events */

#define EVENT_A         1               /* user pressed "A" */
#define EVENT_B         2               /* user pressed "B" */
#define EVENT_C         3               /* user pressed "C" */
#define EVENT_D         4               /* user pressed "D" */
#define EVENT_E         5               /* user pressed "E" */
#define EVENT_F         6               /* user pressed "F" */
#define EVENT_G         7               /* user pressed "G" */
#define EVENT_H         8               /* user pressed "H" */
#define EVENT_I         9               /* user pressed "I" */
#define EVENT_J        10               /* user pressed "J" */
#define EVENT_K        11               /* user pressed "K" */
#define EVENT_L        12               /* user pressed "L" */
#define EVENT_M        13               /* user pressed "M" */
#define EVENT_N        14               /* user pressed "N" */
#define EVENT_O        15               /* user pressed "O" */
#define EVENT_P        16               /* user pressed "P" */
#define EVENT_Q        17               /* user pressed "Q" */
#define EVENT_R        18               /* user pressed "R" */
#define EVENT_S        19               /* user pressed "S" */
#define EVENT_T        20               /* user pressed "T" */
#define EVENT_U        21               /* user pressed "U" */
#define EVENT_V        22               /* user pressed "V" */
#define EVENT_W        23               /* user pressed "W" */
#define EVENT_X        24               /* user pressed "X" */
#define EVENT_Y        25               /* user pressed "Y" */
#define EVENT_Z        26               /* user pressed "Z" */
#define EVENT_BAR      27               /* user pressed space bar */
#define EVENT_LEFT_D   28               /* user pressed left mouse button */
#define EVENT_LEFT_U   29               /* user released l.m.b */
#define EVENT_MIDDLE_D 30               /* user pressed middle button */
#define EVENT_RIGHT_D  31               /* user pressed right button */
#define EVENT_RIGHT_U  32               /* user released r.m.b */
#define EVENT_NETWORK  33               /* incoming network packet */
#define EVENT_INT      34               /* user pressed interrupt key */
#define EVENT_TIMEOUT  35               /* nothing happened! */

int     background;                     /* color of background */
int     foreground;                     /* color of lines and such */
int     color1;                         /* color of 1st line */
int     color2;                         /* color of 2nd line */
int     color3;                         /* color of 3rd line */
int     color4;                         /* color of 4th line */
int     color5;                         /* color of 5th line */
int     iconFore;                       /* icon foreground */
int     iconBack;                       /* icon background */
int     mouseground;                    /* mouse cursor color */
int     highlight;                      /* flash colors */

Window      mwWindow;                   /* parent window */
OpaqueFrame mwWin;                      /* frame for the window */
Window      viewWindow;                 /* view window */
Window      iconWindow;                 /* icon window */
Bitmap      iconMask;                   /* mask for the icon outline */
Pixmap      border_pixmap;              /* Pixmap for the border */
Pixmap      icon_border_pixmap;         /* Pixmap for the icon border */
FontInfo    *fontInfo;                  /* font information */

short gray_bits[16] = {
   0xaaaa, 0x5555, 0xaaaa, 0x5555,
   0xaaaa, 0x5555, 0xaaaa, 0x5555,
   0xaaaa, 0x5555, 0xaaaa, 0x5555,
   0xaaaa, 0x5555, 0xaaaa, 0x5555};

int Xxmin;
int Xymin;
int Xxmax;
int Xymax;

InitWindow(argc,argv)
int argc;
char **argv;
{
   char       *sprintf();
   WindowInfo mwInfo;
   int        defwidth, defheight;
   char       def[128];

   /* Open display first */
   if (!XOpenDisplay(display_name)){
      fprintf(stderr,"%s:Could not open display %s\n",
              progname,display_name);
      exit(1);
   }

   /* Merge Options */
   Merge_Options();

   /* Allocate colors */
   Alloc_Colors();

   /* Find a font */
   if (!(fontInfo = XOpenFont(font_name))){
      fprintf(stderr,"%s:Could not open font %s\n",progname,font_name);
      exit(1);
   }

   /* set up the main window */
   mwWin.bdrwidth = border_width;
   mwWin.border = border_pixmap;
   mwWin.background = XMakeTile(background);
   defwidth = X_DIM;
   defheight= Y_DIM + fontInfo->height * (NLINES + 1);
   sprintf(def,"=$dx%d+300+300",defwidth, defheight);
   mwWindow = XCreate("Kenny_Plotter",progname,geometry,def,&mwWin,
                       defwidth,defheight);

   if (!mwWindow) {
      fprintf(stderr,"XCreateWindow failed");
      exit(1);
   }

   /* set up a subwindow */
   viewWindow = XCreateTransparency(mwWindow,VIEW_X_ORIGIN,VIEW_Y_ORIGIN,
                                             VIEW_X_DIM,VIEW_Y_DIM);

   /* set up the icon */
   XQueryWindow(mwWindow, &mwInfo);
   iconWindow = XCreateWindow(RootWindow,
                       mwInfo.x + (mwInfo.width - icon_width) /2,
                       mwInfo.y + (mwInfo.height - icon_height) /2,
                       icon_width, icon_height, 3, icon_border_pixmap, 0);
   XTileRelative(iconWindow);
   XSetIconWindow(mwWindow, iconWindow);
   iconMask = XStoreBitmap(icon_mask_width, icon_mask_height,
                           icon_mask_bits);

   /* XSelectInput(mwWindow,ExposeWindow | UnmapWindow); */
   XSelectInput(iconWindow,ExposeWindow | UnmapWindow); 
   iconFore = foreground;
   iconBack = background;

   XSelectInput(mwWindow,
                KeyPressed|ButtonPressed|ButtonReleased|
                EnterWindow|LeaveWindow|ExposeWindow|UnmapWindow);
   XMapWindow(viewWindow);
   XMapWindow(mwWindow);
}

/* Initialize and merge the various options */
Merge_Options()
{
   char    *option;

   if (reverse < 0) {
      if ((option = XGetDefault(progname,"ReverseVideo")) != NULL)
         if (strcmp(option,"on")) reverse = 1;
   }
   if (!geometry) {
      option = XGetDefault(progname,"Geometry");
      geometry = option ? option : "=+5+5";
   }
   if (!font_name) {
      /* option = XGetDefault(progname,"BodyFont"); */
      font_name = DEFAULT_FONT;
   }
   if (!border_color) {
      option = XGetDefault(progname,"Border");
      border_color = option ? option : DEFAULT_BORDER_COLOR;
   }
   if (!back_color) {
      option = XGetDefault(progname,"Background");
      back_color = option ? option : DEFAULT_BACK_COLOR;
   }
   if (!fore_color) {
      option = XGetDefault(progname,"Foreground");
      fore_color = option ? option : DEFAULT_FORE_COLOR;
   }
   if (!mouse_color) {
      option = XGetDefault(progname,"Mouse");
      mouse_color = option ? option : DEFAULT_MOUSE_COLOR;
   }
   if (!icon_color) {
      option = XGetDefault(progname,"Icon");
      icon_color = option ? option : DEFAULT_ICON_COLOR;
   }
   if (border_width <= 0) {
      option = XGetDefault(progname,"BorderWidth");
      border_width = option ? atoi(option) : DEFAULT_BORDER_WIDTH;
   }
}


/* Allocate colors */
Alloc_Colors()
{
   Color      cdef;

   /*if DisplayCells are less or equal to 2, then on monochrome display */
   if (DisplayCells() <= 2) {                 
      background =  BlackPixel;
      foreground =  WhitePixel;
      mouseground = WhitePixel;
      color1 = foreground;
      color2 = foreground;
      color3 = foreground;
      color4 = foreground;
      color5 = foreground;
      border_pixmap = XMakePixmap(XStoreBitmap(16,16,gray_bits),
                                         BlackPixel,WhitePixel);
      icon_border_pixmap = border_pixmap;
   } else if (DisplayCells() > 2) {
   /*if DisplayCells are greater than 2, then on color display */
      if (XParseColor(back_color,&cdef) && XGetHardwareColor(&cdef))
         background = cdef.pixel;
      if (XParseColor(fore_color,&cdef) && XGetHardwareColor(&cdef))
         foreground = cdef.pixel;
      if (XParseColor(mouse_color,&cdef)&&XGetHardwareColor(&cdef))
         mouseground = cdef.pixel;
      if (XParseColor("yellow",&cdef)&&XGetHardwareColor(&cdef))
         color1 = cdef.pixel;
      if (XParseColor("cyan",&cdef)&&XGetHardwareColor(&cdef))
         color2 = cdef.pixel;
      if (XParseColor("green",&cdef)&&XGetHardwareColor(&cdef))
         color3 = cdef.pixel;
      if (XParseColor("red",&cdef)&&XGetHardwareColor(&cdef))
         color4 = cdef.pixel;
      if (XParseColor("blue",&cdef)&&XGetHardwareColor(&cdef))
         color5 = cdef.pixel;
      if (XParseColor(border_color,&cdef)&& XGetHardwareColor(&cdef))
         border_pixmap = XMakeTile(cdef.pixel);
      if (XParseColor(icon_color,&cdef) && XGetHardwareColor(&cdef))
         icon_border_pixmap = XMakeTile(cdef.pixel);
   }

   if (reverse) {
      highlight  = background;
      background = foreground;
      foreground = highlight;
      if (mouseground == background) mouseground = foreground;
      color1 = foreground;
      color2 = foreground;
      color3 = foreground;
      color4 = foreground;
      color5 = foreground;
   } else
      highlight = foreground;
}


/* 
 * Event handling. Presents a uniform event interface, and
 * handles all other events in here. Only sees the events
 * that affect it directly. They are:
 *
 *      keypresses 
 *      mouse button clicks
 *
 * All other events should be swallowed by this routine. 
 */

NextEvent()
{
   XEvent  xEvent;
   int     event;

   while (1) {
      if (XPending()) {
         XNextEvent(&xEvent);
         switch (xEvent.type) {
         case KeyPressed:
            event = 0;
            switch(((XKeyEvent *) &xEvent)->detail&0xff) {
            case KC_A:
            case KC_KEYPAD_4:     event = EVENT_A;    return(event);
            case KC_S:
            case KC_KEYPAD_5:     event = EVENT_S;    return(event);
            case KC_D:
            case KC_KEYPAD_6:     event = EVENT_D;    return(event);
            case KC_F:
            case KC_KEYPAD_COMMA: event = EVENT_F;    return(event);
            case KC_BAR:
            case KC_CURSOR_RIGHT: event = EVENT_BAR;  return(event);
            case KC_B:            event = EVENT_B;    return(event);
            case KC_C:            event = EVENT_C;    return(event);
            case KC_E:            event = EVENT_E;    return(event);
            case KC_G:            event = EVENT_G;    return(event);
            case KC_H:            event = EVENT_H;    return(event);
            case KC_I:            event = EVENT_I;    return(event);
            case KC_J:            event = EVENT_J;    return(event);
            case KC_K:            event = EVENT_K;    return(event);
            case KC_L:            event = EVENT_L;    return(event);
            case KC_M:            event = EVENT_M;    return(event);
            case KC_N:            event = EVENT_N;    return(event);
            case KC_O:            event = EVENT_O;    return(event);
            case KC_P:            event = EVENT_P;    return(event);
            case KC_R:            event = EVENT_R;    return(event);
            case KC_T:            event = EVENT_T;    return(event);
            case KC_U:            event = EVENT_U;    return(event);
            case KC_V:            event = EVENT_V;    return(event);
            case KC_W:            event = EVENT_W;    return(event);
            case KC_X:            event = EVENT_X;    return(event);
            case KC_Y:            event = EVENT_Y;    return(event);
            case KC_Z:            event = EVENT_Z;    return(event);
            case KC_Q:
            case KC_DEL:          event = EVENT_INT;  return(event);
            }
            break;

#define RightButton   0
#define MiddleButton  1
#define LeftButton    2

         case ButtonPressed:
            event = 0;
            switch(((XButtonPressedEvent *) &xEvent)->detail&0xff) {
            case RightButton:     event = EVENT_RIGHT_D;   return(event);
            case MiddleButton:    event = EVENT_MIDDLE_D;  return(event);
            case LeftButton:      event = EVENT_LEFT_D;    return(event);
            }
            break;

         case ButtonReleased:
            event = 0;
            switch(((XButtonReleasedEvent *) &xEvent)->detail&0xff) {
            case RightButton:     event = EVENT_RIGHT_U;   return(event);
            case LeftButton:      event = EVENT_LEFT_U;    return(event);
            }
            break;

         case EnterWindow:
            break;

         case LeaveWindow:
            break;

         case ExposeWindow:
            if (((XExposeWindowEvent *)&xEvent)->window == iconWindow) {
               repaintIcon();
               break;
            }
            repaintWindow();
            break;

         case UnmapWindow:
            break;
         }
      }
   }
}

/* Actually put up the pictures on the screen */
repaintWindow()
{
   WindowInfo mwInfo;
   int        Width, Height, FontHeight;

   /* find out about window size */
   FontHeight = fontInfo->height * (NLINES + 1);
   XQueryWindow(mwWindow, &mwInfo);
   Width  = mwInfo.width;
   Height = mwInfo.height - FontHeight;
   if ((Width < X_DIM) || (Height < Y_DIM)) {
      Width  = X_DIM;
      Height = Y_DIM;
   }

   /* graphics routine */
   ShowView(Width,Height); 
}

repaintIcon()
{
XBitmapBitsPut(iconWindow, 0, 0, icon_width, icon_height, icon_bits,
               iconFore, iconBack, iconMask, GXcopy, AllPlanes);
}

RunOps()
{
   int event;

   while(1) {
      event = NextEvent();
      switch (event) {
      case EVENT_LEFT_D   : 
      case EVENT_RIGHT_D  : 
      case EVENT_MIDDLE_D : 
      case EVENT_INT      : XDestroyWindow(mwWindow);
                            XFlush();
                            return; break;
      case EVENT_L  : 
      case EVENT_J  : 
      case EVENT_K  : 
      case EVENT_H  : 
      case EVENT_O  : 
      case EVENT_G  : 
      case EVENT_S :
      case EVENT_T  : 
      case EVENT_BAR :
      default             : break;
      }
   }
}

ShowView(Width, Height)
int      Width, Height;
{
   char *text;
   int  text_len, width;

   XClear(viewWindow);
   text = "Type 'q' or 'Q' to quit";
   text_len = strlen(text);
   width = XStringWidth(text,fontInfo,0,0);
   XText(mwWindow,(X_DIM-width)/2,5,text,text_len,
         fontInfo->id,foreground,background);
   axesX(Width, Height);
   plotX();
}

/* draw the axes */
axesX(Width, Height)
int Width, Height;
{
   char           *sprintf();
   extern char    xlabel[MAXCHAR];
   extern char    ylabel[MAXCHAR];
   extern char    toplabel[MAXCHAR];
   extern int     autorange, equalscale;
   extern double  xmin, xmax, ymin, ymax;

   double vallbl, ratio;
   int    Xxtmp, Xytmp, Xwid, Ywid;
   char   text[100], c;
   int    text_len, text_width, text_xpos, text_ypos, char_width;
   int    label_len;
   int    i;


   /* Initialize */
   Xxmin = BDR_DIM;
   Xymin = BDR_DIM;
   Xwid  = Width  - 2*BDR_DIM;
   Ywid  = Height - 2*BDR_DIM;
   Xxmax = Xxmin + Xwid;
   Xymax = Xymin + Ywid;

   if (equalscale == ON) {
      /* max Xwid = 400, max Ywid = 300 */
      ratio = (ymax-ymin)/(xmax-xmin);
      if (ratio < ((double)Ywid)/((double)Xwid))
         Ywid  = ratio*Xwid;
      else
         Xwid  = Ywid/ratio;
      Xxmax = Xxmin + Xwid;
      Xymax = Xymin + Ywid;
   }

   /* Draw the main axes */
   XLine(viewWindow,Xxmin,Xymin,Xxmin,Xymax,2,2,foreground,GXcopy,AllPlanes);
   XLine(viewWindow,Xxmin,Xymax,Xxmax,Xymax,2,2,foreground,GXcopy,AllPlanes);
   XLine(viewWindow,Xxmax,Xymax,Xxmax,Xymin,2,2,foreground,GXcopy,AllPlanes);
   XLine(viewWindow,Xxmax,Xymin,Xxmin,Xymin,2,2,foreground,GXcopy,AllPlanes);

   /* Top Label */
   text_len = strlen(toplabel);
   text_width = XStringWidth(toplabel,fontInfo,0,0);
   text_xpos = (Xxmax + Xxmin - text_width)/2;
   text_ypos = Xymin - fontInfo->height*3;
   XText(viewWindow,text_xpos,text_ypos,toplabel,text_len,
         fontInfo->id,foreground,background);

   /* Draw the tick marks and labels */
   label_len = 6;
   if (!autorange) {
      plot_Xticks(&label_len);
   } else {
      plot_XAticks(&label_len);
   }

   /* X-Axis Label */
   text_len = strlen(xlabel);
   text_width = XStringWidth(xlabel,fontInfo,0,0);
   text_xpos = (Xxmax + Xxmin - text_width)/2;
   text_ypos = Xymax + fontInfo->height*3;
   XText(viewWindow,text_xpos,text_ypos,xlabel,text_len,
         fontInfo->id,foreground,background);

   /* Y-Axis Label */
   /* Don't know how to rotate, so print character by character */
   char_width = XStringWidth("a",fontInfo,0,0);
   text_len = strlen(ylabel);
   text_width = XStringWidth(ylabel,fontInfo,0,0);
   Xytmp = (Xymax+Xymin)/2 - text_width;
   for (i=0; (c=ylabel[i]) != '\0'; i++) {
      text[0] = c;
      XText(viewWindow,Xxmin-(label_len+6)*fontInfo->width,Xytmp,text,1,
            fontInfo->id,foreground,background);
      Xytmp += 2*char_width;
   }
}


/* Draw the tick marks */
plot_Xticks(label_len)
int *label_len;
{
   char           *sprintf();
   extern int     grid,xticks,yticks,xlog,ylog;
   extern double  xmin, xmax, ymin, ymax;

   double xdx, ydy, xintv, yintv;
   double vallbl;
   int    Xxtmp, Xytmp;
   int    i, llen = 0;
   Vertex vlist[10];

   /* useful data */
   xintv = (Xxmax-Xxmin)/xticks;
   yintv = (Xymax-Xymin)/yticks;
   xdx   = (xmax-xmin)/xticks;
   ydy   = (ymax-ymin)/yticks;

   /* reset the number of ticks */
   if (xlog) {
      xticks = (int)(xmax - xmin - 1e-5) + 1;
      xmax   = xmin + xticks;
   }
   if (ylog) {
      yticks = (int)(ymax - ymin - 1e-5) + 1;
      ymax   = ymin + yticks;
   }

   /* xticks */
   for (i=0; i<=xticks; i++) {
      Xxtmp = Xxmin + (int)(i*xintv);
      XLine(viewWindow,Xxtmp,Xymin,Xxtmp,Xymin+10,
            2,2,foreground,GXcopy,AllPlanes);
      XLine(viewWindow,Xxtmp,Xymax,Xxtmp,Xymax-10,
            2,2,foreground,GXcopy,AllPlanes);

      vallbl = xmin +i*xdx;
      plot_Xxticklabel(vallbl,Xxtmp);
   }

   /* yticks */
   for (i=0; i<=yticks; i++) {
      Xytmp = Xymin + (int)(i*yintv); 
      XLine(viewWindow,Xxmax,Xytmp,Xxmax-10,Xytmp,
            2,2,foreground,GXcopy,AllPlanes);
      XLine(viewWindow,Xxmin,Xytmp,Xxmin+10,Xytmp,
            2,2,foreground,GXcopy,AllPlanes);

      vallbl = ymax -i*ydy;
      plot_Xxticklabel(vallbl,Xytmp,&llen);
   }

   /* return the label length */
   if (llen < *label_len) *label_len = llen;

   /* Work on the grid */
   if (grid == ON) {
      for (i=0; i<=xticks; i++) {
         Xxtmp = Xxmin + (int)(i*xintv);
         vlist[0].x     = Xxtmp;
         vlist[0].y     = Xymin + 10;
         vlist[0].flags = 0;
         vlist[1].x     = Xxtmp;
         vlist[1].y     = Xymax - 10;
         vlist[1].flags = 0;
         XDrawDashed(viewWindow,vlist,2,1,1,
                  foreground,Long_Dotted,GXcopy,AllPlanes);
      }
      for (i=0; i<=yticks; i++) {
         Xytmp = Xymin + (int)(i*yintv); 
         vlist[0].x     = Xxmin + 10;
         vlist[0].y     = Xytmp;
         vlist[0].flags = 0;
         vlist[1].x     = Xxmax - 10;
         vlist[1].y     = Xytmp;
         vlist[1].flags = 0;
         XDrawDashed(viewWindow,vlist,2,1,1,
                  foreground,Long_Dotted,GXcopy,AllPlanes);
      }
   }
}

/* Draw the tick marks and labels with automatic ranging */
plot_XAticks(label_len)
int *label_len;
{
   char           *sprintf();
   extern int     grid,xticks,yticks,xlog,ylog;
   extern double  xmin, xmax, ymin, ymax;

   double xmin2, xmax2, deltx, ymin2, ymax2, delty;
   double xtmp, ytmp, xt, yt, xfactor, yfactor;
   int    Xxtmp, Xytmp;
   int    i, nticks, llen=0;
   Vertex vlist[10];

   /*
    * useful data
    * X = Xxmin + (x-xmin)*(Xxmax-Xxmin)/(xmax-xmin)
    * Y = Xymax + (y-ymin)*(Xymin-Xymax)/(ymax-ymin)
    */
   xfactor = (Xxmax - Xxmin)/(xmax-xmin);
   yfactor = (Xymin - Xymax)/(ymax-ymin);
  
   /* get rounded limits */
   get_autorange(xmin,xmax,ymin,ymax,
                 &xmin2,&xmax2,&deltx,&ymin2,&ymax2,&delty);

   /* xticks */

   /* draw minor tick marks */
   xtmp = xmin2;
   while (xtmp < xmax) {
      for (i=1; i<10; i++) {
         xt = xtmp + i*0.1*deltx;
         if (xlog) xt = xtmp + log10((double)i);
         if ((xt > xmin) && (xt < xmax)) {
            Xxtmp = Xxmin + (xt-xmin)*xfactor;
            if (i==5) {
               XLine(viewWindow,Xxtmp,Xymin,Xxtmp,Xymin+7,
                     2,2,foreground,GXcopy,AllPlanes);
               XLine(viewWindow,Xxtmp,Xymax,Xxtmp,Xymax-7,
                     2,2,foreground,GXcopy,AllPlanes);
            } else {
               XLine(viewWindow,Xxtmp,Xymin,Xxtmp,Xymin+4,
                     2,2,foreground,GXcopy,AllPlanes);
               XLine(viewWindow,Xxtmp,Xymax,Xxtmp,Xymax-4,
                     2,2,foreground,GXcopy,AllPlanes);
            }
         }
      }
      xtmp += deltx;
   }

   /* major tickmarks and accompanying labels */
   xtmp = xmin2 + deltx;
   nticks = 0;
   while (xtmp < xmax-0.0001) {
      Xxtmp = Xxmin + (xtmp-xmin)*xfactor;
      XLine(viewWindow,Xxtmp,Xymin,Xxtmp,Xymin+10,
            2,2,foreground,GXcopy,AllPlanes);
      XLine(viewWindow,Xxtmp,Xymax,Xxtmp,Xymax-10,
            2,2,foreground,GXcopy,AllPlanes);
      plot_Xxticklabel(xtmp,Xxtmp);
      xtmp += deltx;
      nticks ++;
   }

   /* put labels on the min and max points */
   if ((nticks==0) || ((xmin - xmin2) < 0.5*deltx)) {
      plot_Xxticklabel(xmin,Xxmin);
   }

   /* put labels on the min and max points */
   if ((nticks==0) || ((xtmp - xmax) < 0.5*deltx)) {
      plot_Xxticklabel(xmax,Xxmax);
   }

   /* yticks */

   /* draw minor tick marks */
   ytmp = ymin2;
   while (ytmp < ymax) {
      for (i=1; i<10; i++) {
         yt = ytmp + i*0.1*delty;
         if (ylog) yt = ytmp + log10((double)i);
         if ((yt > ymin) && (yt < ymax)) {
            Xytmp = Xymax + (yt-ymin)*yfactor;
            if (i==5) {
               XLine(viewWindow,Xxmax,Xytmp,Xxmax-7,Xytmp,
                     2,2,foreground,GXcopy,AllPlanes);
               XLine(viewWindow,Xxmin,Xytmp,Xxmin+7,Xytmp,
                     2,2,foreground,GXcopy,AllPlanes);
            } else {
               XLine(viewWindow,Xxmax,Xytmp,Xxmax-4,Xytmp,
                     2,2,foreground,GXcopy,AllPlanes);
               XLine(viewWindow,Xxmin,Xytmp,Xxmin+4,Xytmp,
                     2,2,foreground,GXcopy,AllPlanes);
            }
         }
      }
      ytmp += delty;
   }

   /* major tickmarks and accompanying labels */
   ytmp = ymin2 + delty;
   nticks = 0;
   while (ytmp < ymax-0.0001) {
      Xytmp = Xymax + (ytmp-ymin)*yfactor;
      XLine(viewWindow,Xxmax,Xytmp,Xxmax-10,Xytmp,
            2,2,foreground,GXcopy,AllPlanes);
      XLine(viewWindow,Xxmin,Xytmp,Xxmin+10,Xytmp,
            2,2,foreground,GXcopy,AllPlanes);
      plot_Xyticklabel(ytmp,Xytmp,&llen);
      ytmp += delty;
      nticks ++;
   }

   /* put labels on the min and max points */
   if ((nticks==0) || ((ymin - ymin2) < 0.5*delty)) {
      plot_Xyticklabel(ymin,Xymax,&llen);
   }

   /* put labels on the min and max points */
   if ((nticks==0) || ((ytmp - ymax) < 0.5*deltx)) {
      plot_Xyticklabel(ymax,Xymin,&llen);
   }

   /* return the label length */
   if (llen < *label_len) *label_len = llen;

   /* Work on the grid */
   if (grid == ON) {
      xtmp = xmin2 + deltx;
      while (xtmp < xmax) {
         Xxtmp          = Xxmin + (xtmp-xmin)*xfactor;
         vlist[0].x     = Xxtmp;
         vlist[0].y     = Xymin + 10;
         vlist[0].flags = 0;
         vlist[1].x     = Xxtmp;
         vlist[1].y     = Xymax - 10;
         vlist[1].flags = 0;
         XDrawDashed(viewWindow,vlist,2,1,1,
                  foreground,Long_Dotted,GXcopy,AllPlanes);
         xtmp += deltx;
      }

      ytmp = ymin2 + delty;
      while (ytmp < ymax) {
         Xytmp          = Xymax + (ytmp-ymin)*yfactor;
         vlist[0].x     = Xxmin + 10;
         vlist[0].y     = Xytmp;
         vlist[0].flags = 0;
         vlist[1].x     = Xxmax - 10;
         vlist[1].y     = Xytmp;
         vlist[1].flags = 0;
         XDrawDashed(viewWindow,vlist,2,1,1,
                  foreground,Long_Dotted,GXcopy,AllPlanes);
         ytmp += delty;
      }
   }
}

/* plot the tick label on the x-axis */
plot_Xxticklabel(vallbl,Xxtmp)
double vallbl;
int    Xxtmp;
{
   char   *sprintf();
   extern int xlog;
   int    text_len, text_width, text_xpos, text_ypos;
   char   text[MAXCHAR];

   if (xlog) vallbl = pow(10.0,vallbl);
   sprintf(text,"%.3g",vallbl);
   text_len   = strlen(text);
   text_width = XStringWidth(text,fontInfo,0,0);
   text_xpos  = Xxtmp - text_width/2;
   text_ypos  = Xymax + fontInfo->height;
   XText(viewWindow,text_xpos,text_ypos,text,text_len,
            fontInfo->id,foreground,background);
}

/* plot the tick label on the y-axis */
plot_Xyticklabel(vallbl,Xytmp,llen)
double vallbl;
int    Xytmp, *llen;
{
   char   *sprintf();
   extern int ylog;
   int    text_len, text_width, text_xpos, text_ypos, char_width;
   char   text[MAXCHAR];

   if (ylog) vallbl = pow(10.0,vallbl);
   sprintf(text,"%.3g",vallbl);
   text_len   = strlen(text);
   if (text_len > *llen) *llen = text_len;

   text_width = XStringWidth(text,fontInfo,0,0);
   char_width = XStringWidth("a",fontInfo,0,0);
   text_xpos  = Xxmin - text_width - char_width;
   text_ypos  = Xytmp - fontInfo->height;
   XText(viewWindow,text_xpos,text_ypos,text,text_len,
            fontInfo->id,foreground,background);
}

/* draw the plot */
plotX()
{
   extern int     line, linechange, marker, markerchange;
   extern int     linetyp[MAXTYPE],markertype[MAXTYPE], plottypes;
   extern double  xmin, xmax, ymin, ymax;
   extern segmptr segmhead;

   Vertex  vlist[MAXPTS];
   segmptr S;
   nodeptr N;
   double  idx,jdy;
   int     j,linepattern,linecolor,npts,nplt=0;
   int     kl,km;

   idx = (Xxmax-Xxmin)/(xmax-xmin);
   jdy = (Xymax-Xymin)/(ymin-ymax);  /* because of strange x format */

   kl = linetype[nplt];
   km = markertype[nplt];

   S = segmhead;
   for ( ; S!=NULL; S=S->next) {
      N = S->head;
      j = 0;
      /* rescale points, save in array */
      for ( ; N!=NULL && j<MAXPTS; N=N->next) {
         vlist[j].x = (int)((N->x-xmin)*idx) + Xxmin;
         vlist[j].y = (int)((N->y-ymax)*jdy) + Xymin;
         vlist[j].flags = 0;
         j++;
      }
      npts = j;

      /* plot the points */
      if (line == ON && kl!=0) {
         linepattern = linetypX(kl);
         linecolor   = linecolX(kl);
         XDrawDashed(viewWindow,vlist,npts,1,1,
                     linecolor,linepattern,GXcopy,AllPlanes);
      }

      /* plot the markers */
      if (marker == ON || (kl==0 && km==0)) {
         km = km % X_MKR;
         switch (km) {
         case 0 : for (j=0; j < npts; j++) 
                     XLine(viewWindow,
                     vlist[j].x,vlist[j].y,vlist[j].x,vlist[j].y,
                     1,1,foreground,GXcopy,AllPlanes);
                  break;
         case 1 : for (j=0; j < npts; j++) 
                     XLine(viewWindow,
                     vlist[j].x-1,vlist[j].y-1,vlist[j].x-1,vlist[j].y-1,
                     3,3,foreground,GXcopy,AllPlanes);
                  break;
         case 2 : for (j=0; j < npts; j++) {
                     XLine(viewWindow,
                     vlist[j].x-2,vlist[j].y-2,vlist[j].x+2,vlist[j].y+2,
                     1,1,foreground,GXcopy,AllPlanes);
                     XLine(viewWindow,
                     vlist[j].x+2,vlist[j].y-2,vlist[j].x-2,vlist[j].y+2,
                     1,1,foreground,GXcopy,AllPlanes);
                     }
                  break;
         case 3 : for (j=0; j < npts; j++) {
                     XLine(viewWindow,
                     vlist[j].x-2,vlist[j].y-2,vlist[j].x-2,vlist[j].y+2,
                     1,1,foreground,GXcopy,AllPlanes);
                     XLine(viewWindow,
                     vlist[j].x-2,vlist[j].y+2,vlist[j].x+2,vlist[j].y+2,
                     1,1,foreground,GXcopy,AllPlanes);
                     XLine(viewWindow,
                     vlist[j].x+2,vlist[j].y+2,vlist[j].x+2,vlist[j].y-2,
                     1,1,foreground,GXcopy,AllPlanes);
                     XLine(viewWindow,
                     vlist[j].x+2,vlist[j].y-2,vlist[j].x-2,vlist[j].y-2,
                     1,1,foreground,GXcopy,AllPlanes);
                     }
                  break;
         case 4 : for (j=0; j < npts; j++) {
                     XLine(viewWindow,
                     vlist[j].x  ,vlist[j].y-2,vlist[j].x+2,vlist[j].y  ,
                     1,1,foreground,GXcopy,AllPlanes);
                     XLine(viewWindow,
                     vlist[j].x+2,vlist[j].y  ,vlist[j].x  ,vlist[j].y+2,
                     1,1,foreground,GXcopy,AllPlanes);
                     XLine(viewWindow,
                     vlist[j].x  ,vlist[j].y+2,vlist[j].x-2,vlist[j].y  ,
                     1,1,foreground,GXcopy,AllPlanes);
                     XLine(viewWindow,
                     vlist[j].x-2,vlist[j].y  ,vlist[j].x  ,vlist[j].y-2,
                     1,1,foreground,GXcopy,AllPlanes);
                  }
                  break;
         }
      }
      /* increment the plot-type by increasing the array index */
      nplt++;
      if (nplt >= plottypes) nplt = nplt % plottypes;
      /* now look up the line and marker type */
      if (linechange   == ON) kl = linetype[nplt];  
      if (markerchange == ON) km = markertype[nplt];  
   }
}

linetypX(linetyp)
int linetyp;
{
   int linepat;

   linetyp = linetyp % X_LIN;
   switch (linetyp) {
   case 0 : linepat = SolidSpace;  break;
   case 1 : linepat = SolidLine;   break;
   case 2 : linepat = DashedLine;  break;
   case 3 : linepat = DottedLine;  break;
   case 4 : linepat = DotDashLine; break;
   case 5 : linepat = Double_Dot;  break;
   case 6 : linepat = Long_Dash;   break;
   case 7 : linepat = Dot_Dash;    break;
   case 8 : linepat = Long_Dots;   break;
   case 9 : linepat = Short_Dash;  break;
   case 10: linepat = DotDot_Dash; break;
   }
   return(linepat);
}

/* choose a linecolor */
linecolX(linetyp)
int linetyp;
{
   int linecol;

   linetyp = linetyp % X_LIN;
   switch (linetyp) {
      case 0 : linecol = foreground; break;
      case 1 : linecol = color1; break;
      case 2 : linecol = color2; break;
      case 3 : linecol = color3; break;
      case 4 : linecol = color4; break;
      case 5 : linecol = color5; break;
      case 6 : linecol = color1; break;
      case 7 : linecol = color2; break;
      case 8 : linecol = color3; break;
      case 9 : linecol = color4; break;
      case 10: linecol = color5; break;
      default: linecol = foreground; break;
   }
   return(linecol);
}

