/***   plotps.c - POSTSCRIPT subroutines  ***/

#include <stdio.h>
#include <strings.h>
#include <math.h>
#include <time.h>
#include "header.h"

#define PX_DIM   600
#define PY_DIM   500
#define PXL_DIM  800
#define PYL_DIM  600
#define PBDR_DIM 100
#define PS_LIN   11
#define PS_MKR   17

double  Pxmin, Pymin, Pxmax, Pymax;

/* Output Postscript Subroutines */
initps(ips)
FILE *ips;
{
   extern double scale;
   char   time[MAXCHAR];
   double d;

   d = 6*scale;

   /* begin postscript output */
   fprintf(ips,"%c!\n",'%');

   if (landscape) fprintf(ips,"-90 rotate -790 0 translate\n");

   fprintf(ips,"/defFontSizeTop %6.2f def\n",25*scale);
   fprintf(ips,"/defFontSizeSide %6.2f def\n",20*scale);
   fprintf(ips,"/defFontSizeAxes %6.2f def\n",15*scale);
   fprintf(ips,"/defFontSizeText %6.2f def\n",10*scale);

   fprintf(ips,"/UPJ  {} def\n");
   fprintf(ips,"/MIJT  {0 defFontSizeTop -2 div rmoveto} def\n");
   fprintf(ips,"/BOJT  {0 defFontSizeTop neg rmoveto} def\n");
   fprintf(ips,"/MIJS  {0 defFontSizeSide -2 div rmoveto} def\n");
   fprintf(ips,"/BOJS  {0 defFontSizeSide neg rmoveto} def\n");
   fprintf(ips,"/MIJA  {0 defFontSizeAxes -2 div rmoveto} def\n");
   fprintf(ips,"/BOJA  {0 defFontSizeAxes neg rmoveto} def\n");
   fprintf(ips,"/MIJTX {0 defFontSizeText -2 div rmoveto} def\n");
   fprintf(ips,"/BOJTX {0 defFontSizeText neg rmoveto} def\n");

   fprintf(ips,"/LEJ  {} def\n");
   fprintf(ips,"/CEJ  {dup stringwidth pop -2 div 0 rmoveto} def\n");
   fprintf(ips,"/RIJ  {dup stringwidth pop neg 0 rmoveto} def\n");

   fprintf(ips,"/MoveDraw {lineto currentpoint stroke moveto} def\n");

   fprintf(ips,"/markA { %6.2f %6.2f rmoveto\n",d/2,-d/2);
   fprintf(ips,"         %6.2f %6.2f rlineto\n",0.0,  d);
   fprintf(ips,"         %6.2f %6.2f rlineto\n", -d,0.0);
   fprintf(ips,"         %6.2f %6.2f rlineto\n",0.0, -d);
   fprintf(ips,"         %6.2f %6.2f rlineto\n",  d,0.0);
   fprintf(ips,"         gsave\n");
   fprintf(ips,"         1.0 setgray fill\n");
   fprintf(ips,"         grestore\n");
   fprintf(ips,"         stroke } def\n");

   fprintf(ips,"/markB { %6.2f %6.2f rmoveto\n",d/2,-d/2);
   fprintf(ips,"         %6.2f %6.2f rlineto\n",0.0,  d);
   fprintf(ips,"         %6.2f %6.2f rlineto\n", -d,0.0);
   fprintf(ips,"         %6.2f %6.2f rlineto\n",0.0, -d);
   fprintf(ips,"         %6.2f %6.2f rlineto\n",  d,0.0);
   fprintf(ips,"         gsave\n");
   fprintf(ips,"         0.6 setgray fill\n");
   fprintf(ips,"         grestore\n");
   fprintf(ips,"         stroke } def\n");

   fprintf(ips,"/markC { %6.2f %6.2f rmoveto\n",d/2,-d/2);
   fprintf(ips,"         %6.2f %6.2f rlineto\n",0.0,  d);
   fprintf(ips,"         %6.2f %6.2f rlineto\n", -d,0.0);
   fprintf(ips,"         %6.2f %6.2f rlineto\n",0.0, -d);
   fprintf(ips,"         %6.2f %6.2f rlineto\n",  d,0.0);
   fprintf(ips,"         gsave\n");
   fprintf(ips,"         0.0 setgray fill\n");
   fprintf(ips,"         grestore\n");
   fprintf(ips,"         stroke } def\n");

   fprintf(ips,"/markD { %6.2f %6.2f rmoveto\n", 0.0,-d/2);
   fprintf(ips,"         %6.2f %6.2f rlineto\n", d/2, d/2);
   fprintf(ips,"         %6.2f %6.2f rlineto\n",-d/2, d/2);
   fprintf(ips,"         %6.2f %6.2f rlineto\n",-d/2,-d/2);
   fprintf(ips,"         %6.2f %6.2f rlineto\n", d/2,-d/2);
   fprintf(ips,"         gsave\n");
   fprintf(ips,"         1.0 setgray fill\n");
   fprintf(ips,"         grestore\n");
   fprintf(ips,"         stroke } def\n");

   fprintf(ips,"/markE { %6.2f %6.2f rmoveto\n", 0.0,-d/2);
   fprintf(ips,"         %6.2f %6.2f rlineto\n", d/2, d/2);
   fprintf(ips,"         %6.2f %6.2f rlineto\n",-d/2, d/2);
   fprintf(ips,"         %6.2f %6.2f rlineto\n",-d/2,-d/2);
   fprintf(ips,"         %6.2f %6.2f rlineto\n", d/2,-d/2);
   fprintf(ips,"         gsave\n");
   fprintf(ips,"         0.6 setgray fill\n");
   fprintf(ips,"         grestore\n");
   fprintf(ips,"         stroke } def\n");

   fprintf(ips,"/markF { %6.2f %6.2f rmoveto\n", 0.0,-d/2);
   fprintf(ips,"         %6.2f %6.2f rlineto\n", d/2, d/2);
   fprintf(ips,"         %6.2f %6.2f rlineto\n",-d/2, d/2);
   fprintf(ips,"         %6.2f %6.2f rlineto\n",-d/2,-d/2);
   fprintf(ips,"         %6.2f %6.2f rlineto\n", d/2,-d/2);
   fprintf(ips,"         gsave\n");
   fprintf(ips,"         0.0 setgray fill\n");
   fprintf(ips,"         grestore\n");
   fprintf(ips,"         stroke } def\n");

   fprintf(ips,"/markG { %6.2f %6.2f rmoveto\n",-0.50*d,-0.22*d);
   fprintf(ips,"         %6.2f %6.2f rlineto\n",      d,    0.0);
   fprintf(ips,"         %6.2f %6.2f rlineto\n",-0.50*d, 0.66*d);
   fprintf(ips,"         %6.2f %6.2f rlineto\n",-0.50*d,-0.66*d);
   fprintf(ips,"         gsave\n");
   fprintf(ips,"         1.0 setgray fill\n");
   fprintf(ips,"         grestore\n");
   fprintf(ips,"         stroke } def\n");

   fprintf(ips,"/markH { %6.2f %6.2f rmoveto\n",-0.50*d,-0.22*d);
   fprintf(ips,"         %6.2f %6.2f rlineto\n",      d,    0.0);
   fprintf(ips,"         %6.2f %6.2f rlineto\n",-0.50*d, 0.66*d);
   fprintf(ips,"         %6.2f %6.2f rlineto\n",-0.50*d,-0.66*d);
   fprintf(ips,"         gsave\n");
   fprintf(ips,"         0.6 setgray fill\n");
   fprintf(ips,"         grestore\n");
   fprintf(ips,"         stroke } def\n");

   fprintf(ips,"/markI { %6.2f %6.2f rmoveto\n",-0.50*d,-0.22*d);
   fprintf(ips,"         %6.2f %6.2f rlineto\n",      d,    0.0);
   fprintf(ips,"         %6.2f %6.2f rlineto\n",-0.50*d, 0.66*d);
   fprintf(ips,"         %6.2f %6.2f rlineto\n",-0.50*d,-0.66*d);
   fprintf(ips,"         gsave\n");
   fprintf(ips,"         0.0 setgray fill\n");
   fprintf(ips,"         grestore\n");
   fprintf(ips,"         stroke } def\n");

   fprintf(ips,"/markJ { %6.2f %6.2f rmoveto\n",-0.50*d, 0.22*d);
   fprintf(ips,"         %6.2f %6.2f rlineto\n",      d,    0.0);
   fprintf(ips,"         %6.2f %6.2f rlineto\n",-0.50*d,-0.66*d);
   fprintf(ips,"         %6.2f %6.2f rlineto\n",-0.50*d, 0.66*d);
   fprintf(ips,"         gsave\n");
   fprintf(ips,"         1.0 setgray fill\n");
   fprintf(ips,"         grestore\n");
   fprintf(ips,"         stroke } def\n");

   fprintf(ips,"/markK { %6.2f %6.2f rmoveto\n",-0.50*d, 0.22*d);
   fprintf(ips,"         %6.2f %6.2f rlineto\n",      d,    0.0);
   fprintf(ips,"         %6.2f %6.2f rlineto\n",-0.50*d,-0.66*d);
   fprintf(ips,"         %6.2f %6.2f rlineto\n",-0.50*d, 0.66*d);
   fprintf(ips,"         gsave\n");
   fprintf(ips,"         0.6 setgray fill\n");
   fprintf(ips,"         grestore\n");
   fprintf(ips,"         stroke } def\n");

   fprintf(ips,"/markL { %6.2f %6.2f rmoveto\n",-0.50*d, 0.22*d);
   fprintf(ips,"         %6.2f %6.2f rlineto\n",      d,    0.0);
   fprintf(ips,"         %6.2f %6.2f rlineto\n",-0.50*d,-0.66*d);
   fprintf(ips,"         %6.2f %6.2f rlineto\n",-0.50*d, 0.66*d);
   fprintf(ips,"         gsave\n");
   fprintf(ips,"         0.0 setgray fill\n");
   fprintf(ips,"         grestore\n");
   fprintf(ips,"         stroke } def\n");

   fprintf(ips,"/markM { %6.2f %6.2f rmoveto\n", 0.0,-d/2);
   fprintf(ips,"         %6.2f %6.2f rlineto\n", 0.0,   d);
   fprintf(ips,"         %6.2f %6.2f rmoveto\n",-d/2,-d/2);
   fprintf(ips,"         %6.2f %6.2f rlineto\n",   d, 0.0);
   fprintf(ips,"         %6.2f %6.2f rlineto\n",-d/2, 0.0);
   fprintf(ips,"         currentpoint %f 0 360 arc \n",d/2);
   fprintf(ips,"         gsave\n");
   fprintf(ips,"         1.0 setgray fill\n");
   fprintf(ips,"         grestore\n");
   fprintf(ips,"         stroke } def\n");

   fprintf(ips,"/markN { %6.2f %6.2f rmoveto\n", 0.0,-d/2);
   fprintf(ips,"         %6.2f %6.2f rlineto\n", 0.0,   d);
   fprintf(ips,"         %6.2f %6.2f rmoveto\n",-d/2,-d/2);
   fprintf(ips,"         %6.2f %6.2f rlineto\n",   d, 0.0);
   fprintf(ips,"         %6.2f %6.2f rlineto\n",-d/2, 0.0);
   fprintf(ips,"         currentpoint %f 0 360 arc \n",d/2);
   fprintf(ips,"         gsave\n");
   fprintf(ips,"         0.6 setgray fill\n");
   fprintf(ips,"         grestore\n");
   fprintf(ips,"         stroke } def\n");

   fprintf(ips,"/markO { %6.2f %6.2f rmoveto\n", 0.0,-d/2);
   fprintf(ips,"         %6.2f %6.2f rlineto\n", 0.0,   d);
   fprintf(ips,"         %6.2f %6.2f rmoveto\n",-d/2,-d/2);
   fprintf(ips,"         %6.2f %6.2f rlineto\n",   d, 0.0);
   fprintf(ips,"         %6.2f %6.2f rlineto\n",-d/2, 0.0);
   fprintf(ips,"         currentpoint %f 0 360 arc \n",d/2);
   fprintf(ips,"         gsave\n");
   fprintf(ips,"         0.0 setgray fill\n");
   fprintf(ips,"         grestore\n");
   fprintf(ips,"         stroke } def\n");

   fprintf(ips,"/markP { %6.2f %6.2f rmoveto\n",-d/2,-d/2);
   fprintf(ips,"         %6.2f %6.2f rlineto\n",   d,   d);
   fprintf(ips,"         %6.2f %6.2f rmoveto\n",  -d, 0.0);
   fprintf(ips,"         %6.2f %6.2f rlineto\n",   d,  -d);
   fprintf(ips,"         stroke } def\n");

   fprintf(ips,"/markQ { %6.2f %6.2f rmoveto\n",-0.5,0.0);
   fprintf(ips,"         %6.2f %6.2f rlineto\n", 1.0,0.0);
   fprintf(ips,"         stroke } def\n");

   /* print the time */
   get_localtime(time);
   fprintf(ips,"/Times-Bold findfont\n");
   fprintf(ips,"10 scalefont setfont\n");
   if (landscape)
      fprintf(ips,"%6.2f %6.2f moveto ",30.0,570.0);
   else 
      fprintf(ips,"%6.2f %6.2f moveto ",30.0,750.0);
   fprintf(ips,"(%s) show\n",time);
}

/* draw the axes */
axesps(ips)
FILE *ips;
{
   char           *sprintf();
   extern char    xlabel[MAXCHAR];
   extern char    ylabel[MAXCHAR];
   extern char    toplabel[MAXCHAR];
   extern int     autorange, equalscale, clip, linechange, markerchange;
   extern double  scale;
   extern double  xmin, xmax, ymin, ymax;

   double xwid,ywid,ratio;
   int    label_len,xbwid,dist;

   /* Initialize */
   Pxmin = PBDR_DIM;
   Pymin = PBDR_DIM;
   xbwid = 2.0*PBDR_DIM;
   if (linechange || markerchange) xbwid = 2.5*PBDR_DIM;
   xwid  = PX_DIM - xbwid;               /* 350 */
   ywid  = PY_DIM - 2*PBDR_DIM;          /* 300 */
   if (landscape) {
      xwid  = PXL_DIM - xbwid;           /* 550 */
      ywid  = PYL_DIM - 2*PBDR_DIM;      /* 400 */
   }
   Pxmax = Pxmin + scale*xwid;
   Pymax = Pymin + scale*ywid; 

   if (equalscale == ON) {
      /* max xwid = 400, max ywid = 600 */
      ratio = (ymax-ymin)/(xmax-xmin);
      if (ratio <= (ywid/xwid)) {
         /* decrease ywid : ywid  = ratio*xwid */
         Pymax = Pymax - scale*0.5*(ywid - ratio*xwid);
         Pymin = Pymin + scale*0.5*(ywid - ratio*xwid);
      } else {
         /* decrease xwid : xwid  = ywid/ratio */
         Pxmax = Pxmax - scale*0.5*(xwid - ywid/ratio);
         Pxmin = Pxmin + scale*0.5*(xwid - ywid/ratio);
      }
   }

   /* define the axes */
   fprintf(ips,"/rectanglepath {\n");
   fprintf(ips,"newpath\n  %6.2f %6.2f moveto\n",Pxmin,Pymin);
   fprintf(ips,"  0.0 %6.2f rlineto\n",Pymax-Pymin);
   fprintf(ips,"  %6.2f 0.0 rlineto\n",Pxmax-Pxmin);
   fprintf(ips,"  0.0 %6.2f rlineto\n",Pymin-Pymax);
   fprintf(ips,"closepath } def\n");
   fprintf(ips,"%6.2f setlinewidth\nrectanglepath stroke\n",2*scale);

   /* Top Label */
   fprintf(ips,"/Times-Bold findfont\n");
   fprintf(ips,"defFontSizeTop scalefont setfont\n");
   fprintf(ips,"  %6.2f %6.2f moveto ",0.5*(Pxmax+Pxmin),Pymax+25*scale);
   fprintf(ips,"(%s) UPJ CEJ show\n",toplabel);

   /* Draw the tick marks and labels */
   fprintf(ips,"%6.2f setlinewidth\n",1*scale);
   label_len = 6;
   if (!autorange) {
      plot_PSticks(ips,&label_len);
   } else {
      plot_PSAticks(ips,&label_len);
   }

   /* X-axis Label */
   fprintf(ips,"/Times-Bold findfont\n");
   fprintf(ips,"defFontSizeSide scalefont setfont\n");
   fprintf(ips,"  %6.2f %6.2f moveto ",0.5*(Pxmax+Pxmin),Pymin-20*scale);
   fprintf(ips,"(%s) BOJS CEJ show\n",xlabel);

   /* Y-axis Label */
   fprintf(ips,"/Times-Bold findfont\n");
   fprintf(ips,"defFontSizeSide scalefont setfont\n");
   fprintf(ips,"/Ylabel {");
   dist = 10*label_len + 5;
   if (dist > 65) dist = 65;
   fprintf(ips,"%6.2f %6.2f moveto ",0.5*(Pymax+Pymin),
           dist*scale-Pxmin);
   fprintf(ips,"(%s) UPJ CEJ show ",ylabel);
   fprintf(ips,"} def\n  90 rotate\n  Ylabel\n  -90 rotate\n"); 

   /* plot the line labels */
   if (linechange || markerchange)
      plot_ps_labels(ips);

   /* clip if necessary */
   fprintf(ips,"%6.2f setlinewidth\n",0.5*scale);
   if (clip) fprintf(ips,"rectanglepath clip\n");

}

/* Draw the tick marks and labels */
plot_PSticks(ips,label_len)
FILE *ips;
int  *label_len;
{
   char           *sprintf();
   extern int     grid,xticks,yticks,xlog,ylog;
   extern double  scale;
   extern double  xmin, xmax, ymin, ymax;

   double xdx,ydy,xintv,yintv;
   double Pxtmp,Pytmp;
   double vallbl;
   int    i, llen = 0;

   /* reset the number of ticks */
   if (xlog) {
      xticks = (int)(xmax - xmin - 1e-5) + 1;
      xmax   = xmin + xticks;
   }
   if (ylog) {
      yticks = (int)(ymax - ymin - 1e-5) + 1;
      ymax   = ymin + yticks;
   }

   /* Useful data */
   xdx   = (xmax-xmin)/xticks;
   ydy   = (ymax-ymin)/yticks;
   xintv = (Pxmax-Pxmin)/xticks;
   yintv = (Pymax-Pymin)/yticks;

   /* Tick Marks on the X-axis */
   fprintf(ips,"/Times-Roman findfont\n");
   fprintf(ips,"defFontSizeAxes scalefont setfont\n");
   for (i=0; i<=xticks; i++) {
      Pxtmp = Pxmin + xintv*i;
      Pytmp = Pymin;
      fprintf(ips,"newpath %6.2f %6.2f moveto\n",Pxtmp,Pytmp);
      fprintf(ips,"  0 %6.2f rlineto stroke\n",7*scale);

      vallbl = xmin + i*xdx;
      plot_PSxticklabel(ips,vallbl,Pxtmp);

      Pytmp = Pymax;
      fprintf(ips,"newpath %6.2f %6.2f moveto\n",Pxtmp,Pytmp);
      fprintf(ips,"  0 %6.2f rlineto stroke\n",-7*scale);
   }

   /* Tick Marks on the Y-axis */
   for (i=0; i<=yticks; i++) {
      Pytmp = Pymin + yintv*i;
      Pxtmp = Pxmin;
      fprintf(ips,"newpath %6.2f %6.2f moveto\n",Pxtmp,Pytmp);
      fprintf(ips,"  %6.2f 0 rlineto stroke\n",7*scale);

      vallbl = ymin + i*ydy;
      plot_PSyticklabel(ips,vallbl,Pytmp,&llen);

      Pxtmp = Pxmax;
      fprintf(ips,"newpath %6.2f %6.2f moveto\n",Pxtmp,Pytmp);
      fprintf(ips,"  %6.2f 0 rlineto stroke\n",-7*scale);
   }

   /* return the label length */
   if (llen < *label_len) *label_len = llen;

   /* draw the grid */
   if (grid == ON) {
      for (i=1; i<xticks; i++) {
         Pxtmp = Pxmin + xintv*i;
         Pytmp = Pymax - 8*scale;
         fprintf(ips,"[1 4] 1 setdash\n");
         fprintf(ips,"%6.2f setlinewidth 0.0 setgray\n",0.5*scale);
         fprintf(ips,"newpath %6.2f %6.2f moveto ",Pxtmp,Pytmp);
         fprintf(ips,"0 %6.2f rlineto stroke\n",Pymin-Pymax+16*scale);
         fprintf(ips,"%6.2f setlinewidth 0.0 setgray\n",2.0*scale);
         linetypps(ips,1);
      }
      for (i=1; i<yticks; i++) {
         Pytmp = Pymin + yintv*i;
         Pxtmp = Pxmax - 8*scale;
         fprintf(ips,"[1 4] 1 setdash\n");
         fprintf(ips,"%6.2f setlinewidth 0.0 setgray\n",0.5*scale);
         fprintf(ips,"newpath %6.2f %6.2f moveto ",Pxtmp,Pytmp);
         fprintf(ips,"%6.2f 0 rlineto stroke\n",Pxmin-Pxmax+16*scale);
         fprintf(ips,"%6.2f setlinewidth 0.0 setgray\n",2.0*scale);
         linetypps(ips,1);
      }
   }
}

/* Draw the tick marks and labels with automatic ranging */
plot_PSAticks(ips,label_len)
FILE *ips;
int  *label_len;
{
   char           *sprintf();
   double         log10();
   extern int     grid,xticks,yticks,xlog,ylog;
   extern double  scale;
   extern double  xmin, xmax, ymin, ymax;

   double xmin2, xmax2, deltx, ymin2, ymax2, delty;
   double xtmp, ytmp, xt, yt, xfactor, yfactor;
   double Pxtmp, Pytmp;
   int    i, nticks, llen = 0;

   /*
    * useful data
    * X = Pxmin + (x-xmin)*(Pxmax-Pxmin)/(xmax-xmin)
    * Y = Pymin + (y-ymin)*(Pymax-Pymin)/(ymax-ymin)
    */
   xfactor = (Pxmax - Pxmin)/(xmax-xmin);
   yfactor = (Pymax - Pymin)/(ymax-ymin);

   /* get rounded limits */
   get_autorange(xmin,xmax,ymin,ymax,
                 &xmin2,&xmax2,&deltx,&ymin2,&ymax2,&delty);

   /* Tick Marks on the X-axis */

   /* draw minor tick marks */
   xtmp = xmin2;
   while (xtmp < xmax) {
      for (i=1; i<10; i++) {
         xt = xtmp + i*0.1*deltx;
         if (xlog) xt = xtmp + log10((double)i);
         if ((xt > xmin) && (xt < xmax)) {
            Pxtmp = Pxmin + (xt-xmin)*xfactor;
            if (i==5) {
               fprintf(ips,"newpath %6.2f %6.2f moveto\n",Pxtmp,Pymin);
               fprintf(ips,"  0 %6.2f rlineto stroke\n", 5*scale);
               fprintf(ips,"newpath %6.2f %6.2f moveto\n",Pxtmp,Pymax);
               fprintf(ips,"  0 %6.2f rlineto stroke\n",-5*scale);
            } else {
               fprintf(ips,"newpath %6.2f %6.2f moveto\n",Pxtmp,Pymin);
               fprintf(ips,"  0 %6.2f rlineto stroke\n", 3*scale);
               fprintf(ips,"newpath %6.2f %6.2f moveto\n",Pxtmp,Pymax);
               fprintf(ips,"  0 %6.2f rlineto stroke\n",-3*scale);
            }
         }
      }
      xtmp += deltx;
   }

   /* major tickmarks and accompanying labels */
   fprintf(ips,"/Times-Roman findfont\n");
   fprintf(ips,"defFontSizeAxes scalefont setfont\n");
   xtmp = xmin2 + deltx;
   nticks = 0;
   while (xtmp < xmax-0.0001) {
      Pxtmp = Pxmin + (xtmp-xmin)*xfactor;

      fprintf(ips,"newpath %6.2f %6.2f moveto\n",Pxtmp,Pymin);
      fprintf(ips,"  0 %6.2f rlineto stroke\n", 7*scale);

      plot_PSxticklabel(ips,xtmp,Pxtmp);

      fprintf(ips,"newpath %6.2f %6.2f moveto\n",Pxtmp,Pymax);
      fprintf(ips,"  0 %6.2f rlineto stroke\n",-7*scale);

      xtmp += deltx;
      nticks ++;
   }

   /* put labels on the min and max points */
   if ((nticks==0) || ((xmin - xmin2) < 0.5*deltx)) {
      plot_PSxticklabel(ips,xmin,Pxmin);
   }

   /* put labels on the min and max points */
   if ((nticks==0) || ((xtmp - xmax) < 0.5*deltx)) {
      plot_PSxticklabel(ips,xmax,Pxmax);
   }

   /* Tick Marks on the Y-axis */

   /* draw minor tick marks */
   ytmp = ymin2;
   while (ytmp < ymax) {
      for (i=1; i<10; i++) {
         yt = ytmp + i*0.1*delty;
         if (ylog) yt = ytmp + log10((double)i);
         if ((yt > ymin) && (yt < ymax)) {
            Pytmp = Pymin + (yt-ymin)*yfactor;
            if (i==5) {
               fprintf(ips,"newpath %6.2f %6.2f moveto\n",Pxmin,Pytmp);
               fprintf(ips,"  %6.2f 0 rlineto stroke\n", 5*scale);
               fprintf(ips,"newpath %6.2f %6.2f moveto\n",Pxmax,Pytmp);
               fprintf(ips,"  %6.2f 0 rlineto stroke\n",-5*scale);
            } else {
               fprintf(ips,"newpath %6.2f %6.2f moveto\n",Pxmin,Pytmp);
               fprintf(ips,"  %6.2f 0 rlineto stroke\n", 3*scale);
               fprintf(ips,"newpath %6.2f %6.2f moveto\n",Pxmax,Pytmp);
               fprintf(ips,"  %6.2f 0 rlineto stroke\n",-3*scale);
            }
         }
      }
      ytmp += delty;
   }

   /* major tickmarks and accompanying labels */
   ytmp = ymin2 + delty;
   nticks = 0;
   while (ytmp < ymax-0.0001) {
      Pytmp = Pymin + (ytmp-ymin)*yfactor;

      fprintf(ips,"newpath %6.2f %6.2f moveto\n",Pxmin,Pytmp);
      fprintf(ips,"  %6.2f 0 rlineto stroke\n", 7*scale);

      plot_PSyticklabel(ips,ytmp,Pytmp,&llen);

      fprintf(ips,"newpath %6.2f %6.2f moveto\n",Pxmax,Pytmp);
      fprintf(ips,"  %6.2f 0 rlineto stroke\n",-7*scale);

      ytmp += delty;
      nticks ++;
   }

   /* put labels on the min and max points */
   if ((nticks==0) || ((ymin - ymin2) < 0.5*delty)) {
      plot_PSyticklabel(ips,ymin,Pymin,&llen);
   }

   /* put labels on the min and max points */
   if ((nticks==0) || ((ytmp - ymax) < 0.5*deltx)) {
      plot_PSyticklabel(ips,ymax,Pymax,&llen);
   }

   /* return the label length */
   if (llen < *label_len) *label_len = llen;

   /* draw the grid */
   if (grid == ON) {
      xtmp = xmin2 + deltx;
      while (xtmp < xmax-0.0001) {
         Pxtmp = Pxmin + (xtmp-xmin)*xfactor;
         Pytmp = Pymax - 8*scale;
         fprintf(ips,"[1 4] 1 setdash\n");
         fprintf(ips,"%6.2f setlinewidth 0.0 setgray\n",0.5*scale);
         fprintf(ips,"newpath %6.2f %6.2f moveto ",Pxtmp,Pytmp);
         fprintf(ips,"0 %6.2f rlineto stroke\n",Pymin-Pymax+16*scale);
         fprintf(ips,"%6.2f setlinewidth 0.0 setgray\n",2.0*scale);
         linetypps(ips,1);
         xtmp += deltx;
      }
      ytmp = ymin2 + delty;
      while (ytmp < ymax-0.0001) {
         Pytmp = Pymin + (ytmp-ymin)*yfactor;
         Pxtmp = Pxmax - 8*scale;
         fprintf(ips,"[1 4] 1 setdash\n");
         fprintf(ips,"%6.2f setlinewidth 0.0 setgray\n",0.5*scale);
         fprintf(ips,"newpath %6.2f %6.2f moveto ",Pxtmp,Pytmp);
         fprintf(ips,"%6.2f 0 rlineto stroke\n",Pxmin-Pxmax+16*scale);
         fprintf(ips,"%6.2f setlinewidth 0.0 setgray\n",2.0*scale);
         linetypps(ips,1);
         ytmp += delty;
      }
   }
}

/* plot the tick label on the x-axis */
plot_PSxticklabel(ips,vallbl,Pxtmp)
FILE *ips;
double vallbl, Pxtmp;
{
   char   *sprintf();
   extern int     xlog;

   char   label[MAXCHAR];

   if (xlog) vallbl = pow(10.0,vallbl);
   sprintf(label,"%.3g",vallbl);
   fprintf(ips,"  %6.2f %6.2f moveto",Pxtmp,Pymin);
   fprintf(ips,"  (%s) BOJA CEJ show\n",label);
}

/* plot the tick label on the y-axis */
plot_PSyticklabel(ips,vallbl,Pytmp,llen)
FILE *ips;
double vallbl, Pytmp;
int    *llen;
{
   char   *sprintf();
   extern int     xlog;

   int    label_len;
   char   label[MAXCHAR];

   if (ylog) vallbl = pow(10.0,vallbl);
   sprintf(label,"%.3g",vallbl);
   label_len = strlen(label);
   if (label_len > *llen) *llen = label_len;
   fprintf(ips,"  %6.2f %6.2f moveto",Pxmin-5*scale,Pytmp+3*scale);
   fprintf(ips,"  (%s) MIJA RIJ show\n",label);
}

/* draw the plot */
plotps(ips)
FILE *ips;
{
   char mkr_to_char();

   extern int     line, marker;
   extern int     linechange, markerchange;
   extern int     linetype[MAXTYPE], markertype[MAXTYPE], plottypes;
   extern double  xmin, xmax, ymin, ymax;
   extern segmptr segmhead;

   segmptr S;
   double  idx,jdy;
   int     km, kl, nplt=0;

   idx = (Pxmax-Pxmin)/(xmax-xmin);
   jdy = (Pymax-Pymin)/(ymax-ymin);

   kl = linetype[nplt];
   km = markertype[nplt];

   S = segmhead;
   for ( ; S!=NULL; S=S->next) {
     
      /* plot markers */
      km = km % PS_MKR;
      plot_ps_markers(ips,S,idx,jdy,kl,km);

      /* plot the points */
      plot_ps_points(ips,S,idx,jdy,kl);

      /* increment the plot-type by increasing the array index */
      nplt++;
      if (nplt >= plottypes) nplt = nplt % plottypes;
      /* now look up the line and marker type */
      if (linechange   == ON) kl = linetype[nplt];
      if (markerchange == ON) km = markertype[nplt];
   }
}

/* plot markers */
plot_ps_markers(ips,S,idx,jdy,kl,km)
FILE    *ips;
segmptr S;
double  idx, jdy;
int     kl, km;
{
   extern  int marker;
   nodeptr N;
   double  x,y;
   char    charmkr;

   if (marker==ON || (kl==0 && km==0)) { 
      charmkr = mkr_to_char(km);
      linetypps(ips,1);
      fprintf(ips,"newpath");
      for (N=S->head; N!=NULL; N=N->next) {
         x = (N->x-xmin)*idx + Pxmin;
         y = (N->y-ymin)*jdy + Pymin;
         fprintf(ips," %8.2f %8.2f moveto mark%1c\n",x,y,charmkr);
      }
   }
}

/* plot the points */
plot_ps_points(ips,S,idx,jdy,kl)
FILE    *ips;
segmptr S;
double  idx, jdy;
int     kl;
{
   extern  int line;
   nodeptr N;
   int     count;
   double  x,y;

   if (line == ON && kl!=0) {
      linetypps(ips,kl);
      count = 0;
      for (N=S->head; N!=NULL; N=N->next) {
         x = (N->x-xmin)*idx + Pxmin;
         y = (N->y-ymin)*jdy + Pymin;
         if (count == 0) {
            fprintf(ips,"newpath\n  %8.2f %8.2f moveto\n",x,y);
            count++;
         } else if (count > 500) {
            fprintf(ips,"  %8.2f %8.2f lineto\n",x,y);
            fprintf(ips,"stroke\n");
            N = N->prev;
            count = 0;
         } else {
            fprintf(ips,"  %8.2f %8.2f lineto\n",x,y);
            count++;
         }
      } 
      fprintf(ips,"stroke\n");
   }
}

/* plot labels */
plot_ps_labels(ips)
FILE *ips;
{
   nodeptr insert_node();
   segmptr insert_segm();
   void    delete_segm();
   extern  int linechange, markerchange;
   extern  int linetyp[MAXTYPE],markertype[MAXTYPE],plottypes;
   extern  char linelabel[MAXTYPE][MAXCHAR];
   extern  double xmin, xmax, ymin, ymax;
   segmptr S;
   nodeptr N;
   int     i,kl,km,ncurves,nlines,nplt=0;
   double  idx, jdy, x, y;

   /* count no of curves */
   ncurves = 0;
   for (S=segmhead; S!=NULL; S=S->next) ncurves++;

   /* no of lines to be plotted */
   nlines = (plottypes < ncurves) ? plottypes : ncurves;
  
   /* scaling factors */
   idx = (Pxmax-Pxmin)/(xmax-xmin);
   jdy = (Pymax-Pymin)/(ymax-ymin);

   /* set the points */
   S = insert_segm();
   N = insert_node(xmax+10.0/idx,ymax-20.0/jdy,0.0,S);
   N = insert_node(xmax+40.0/idx,ymax-20.0/jdy,0.0,S);

   /* reset the linetype and font */
   fprintf(ips,"/Times-Roman findfont\n");
   fprintf(ips,"defFontSizeText scalefont setfont\n");
   fprintf(ips,"%6.2f setlinewidth 0.0 setgray\n",0.5*scale);

   /* draw only 1st 16 marks */
   for (i=0; i<nlines && i<16; i++) {
      /* set linetype and markertype */   
      kl = linetype[nplt];
      km = markertype[nplt];
      
      /* draw marker */
      plot_ps_markers(ips,S,idx,jdy,kl,km);

      /* plot the points */
      plot_ps_points(ips,S,idx,jdy,kl);

      /* draw linelabel */
      x = (S->tail->x-xmin)*idx + Pxmin + 10;
      y = (S->tail->y-ymin)*jdy + Pymin;
      fprintf(ips,"newpath  %6.2f %6.2f moveto",x,y);
      fprintf(ips,"  (%s) MIJTX LEJ show\n",linelabel[nplt]);

      /* increment the plot-type by increasing the array index */
      nplt++;
      if (nplt >= plottypes) nplt = nplt % plottypes;
      /* now look up the line and marker type */
      if (linechange   == ON) kl = linetype[nplt];
      if (markerchange == ON) km = markertype[nplt];

      /* change ypos */
      S->head->y       -= 20.0/jdy;
      S->head->next->y -= 20.0/jdy;
   }

   /* delete the segm */
   delete_segm(S);
}

/* find the marker type */
char mkr_to_char(num)
int num;
{
    char ch;

    switch (num) {
    case  0 : ch = 'Q'; break;
    case  1 : ch = 'A'; break;
    case  2 : ch = 'B'; break;
    case  3 : ch = 'C'; break;
    case  4 : ch = 'D'; break;
    case  5 : ch = 'E'; break;
    case  6 : ch = 'F'; break;
    case  7 : ch = 'G'; break;
    case  8 : ch = 'H'; break;
    case  9 : ch = 'I'; break;
    case 10 : ch = 'J'; break;
    case 11 : ch = 'K'; break;
    case 12 : ch = 'L'; break;
    case 13 : ch = 'M'; break;
    case 14 : ch = 'N'; break;
    case 15 : ch = 'O'; break;
    case 16 : ch = 'P'; break;
    default : ch = 'A'; break;
    }
    return(ch);
}

/* find the linetype */
linetypps(ips,line)
FILE *ips;
int line; 
{
   line = line % PS_LIN;

   switch (line) {
   case 0 : fprintf(ips,"[0 1] 0 setdash\n"); break;    /* No Line */
   case 1 : fprintf(ips,"[] 0 setdash\n"); break;       /* Solid */
   case 2 : fprintf(ips,"[4 3] 0 setdash\n"); break;    /* Dashed */
   case 3 : fprintf(ips,"[1 2] 1 setdash\n"); break;    /* Dotted */
   case 4 : fprintf(ips,"[6 2 2 2] 0 setdash\n"); break;/* Dot Dash */
   case 5 : fprintf(ips,"[2 2 2 6] 0 setdash\n"); break;/* Double Dot */
   case 6 : fprintf(ips,"[8 2] 0 setdash\n"); break;    /* Long Dash */
   case 7 : fprintf(ips,"[9 6 2 6] 0 setdash\n"); break;/* Dot Dash */
   case 8 : fprintf(ips,"[2 2] 0 setdash\n"); break;    /* Long Dots */
   case 9 : fprintf(ips,"[6 10] 0 setdash\n"); break;   /* Short Dash */
   case 10: fprintf(ips,"[3 3 3 3 6 3] 0 setdash\n");
            break;                                      /* DotDot Dash */
   default: fprintf(ips,"[] 0 setdash\n"); break;
   }
}

/* End procedure */
endgrps(ips)
FILE *ips;
{
   fprintf(ips,"showpage\n");

   /* Close the file */
   fclose(ips);
}


