/***   initsun.c - sun subroutines   ***/

#include <usercore.h>
#include <stdio.h>
#include <strings.h>
#include <math.h>
#include "header.h"

/* @(#)demolib.h 1.7 88/02/07 SMI */
/* for use with demolib.c and Sun demo programs */

struct  vwsurf  *get_color_surface(/* (struct vwsurf *), argv */);
struct  vwsurf  *get_view_surface(/* (struct vwsurf *), argv */);
struct  vwsurf  *get_surface(/* (struct vwsurf *), argv */);

/* storage allocation for vwsurf to pass into get_surface routines */
static  struct  vwsurf  Core_vwsurf;

/* actual vwsurf handle */
struct  vwsurf  *our_surface = &Core_vwsurf;

#define NUMLINES        343     /* number of vectors in a design */
#define MAPSIZE         114
#define SX_DIM          600
#define SY_DIM          500
#define SBDR_DIM        100
#define SUN_MKR          10
#define SUN_LIN           4
double  Sxmin,Symin,Sxmax,Symax;
static  float red[MAPSIZE+1], grn[MAPSIZE+1], blu[MAPSIZE+1];

/* initialize the surface */
init_surf(argv)
        char *argv[];
{
        get_view_surface(our_surface,argv);
        our_surface->cmapsize = 128;
        our_surface->cmapname[0] = '\0';
        if(initialize_core(BASIC, SYNCHRONOUS, TWOD))
                exit(1);
        if(initialize_view_surface(our_surface,FALSE))
                exit(2);
        if(select_view_surface(our_surface))
                exit(3);
}

/* define the color map */
make_maps()
{
        int i;

        red[0] = 0.0;                           /* background color */
        grn[0] = 0.0;
        blu[0] = 0.0;

        for (i=0; i<19; i++) {
                red[i +   1] = 0.99;            /* ramp to yellow */
                grn[i +   1] = 0.055*i;
                blu[i +   1] = 0.0;

                red[i +  20] = 0.99-0.055*i;    /* ramp to green */
                grn[i +  20] = 0.99;
                blu[i +  20] = 0.0;

                red[i +  39] = 0.0;             /* ramp to turqouise */
                grn[i +  39] = 0.99;
                blu[i +  39] = 0.055*i;

                red[i +  58] = 0.0;             /* ramp to blue */
                grn[i +  58] = 0.99-0.055*i;
                blu[i +  58] = 0.99;

                red[i +  77] = 0.055*i;         /* ramp to violet */
                grn[i +  77] = 0.0;
                blu[i +  77] = 0.99;

                red[i +  96] = 0.99;            /* ramp to red */
                grn[i +  96] = 0.0;
                blu[i +  96] = 0.99-0.055*i;
        }

        define_color_indices(our_surface,0,MAPSIZE,red,grn,blu);

} /* end of make_maps() */

close_surf()
{
        deselect_view_surface(our_surface);
        terminate_core();
} /* end of close_surf */

/* initialize */
initsun(argc,argv)
int argc;
char *argv[];
{

        init_surf(argv);       /* Surface Initialization Routine */
        make_maps();           /* ColorMap Initialization Routine */
        set_ndc_space_2(1.0,1.0);
        set_viewport_2(0.0,1.0,0.0,1.0);
        set_text_index(20);
        set_charprecision(CHARACTER);
        set_font(ROMAN);
        initialize_device(LOCATOR,1);
        set_echo(LOCATOR,1,1);
}

/* draw the axes */
axessun()
{
   char            *sprintf();
   extern char     xlabel[MAXCHAR];
   extern char     ylabel[MAXCHAR];
   extern char     toplabel[MAXCHAR];
   extern int      autorange,equalscale,clip,linechange,markerchange;
   extern double   xmin, xmax, ymin, ymax;

   double xwid,ywid;
   float  cdx, cdy;
   double Sxpos, Sypos;
   char   guide[MAXCHAR];

   /* Initialize */
   Sxmin = SBDR_DIM;
   Symin = SBDR_DIM;
   xwid  = SX_DIM - 2*SBDR_DIM;
   ywid  = SY_DIM - 2*SBDR_DIM;
   Sxmax = Sxmin + xwid;
   Symax = Symin + ywid; 
   if (equalscale == ON) Symax = Symin + (ymax-ymin)/(xmax-xmin)*xwid;
   set_linewidth(1.00);
   if (Sxmax >= Symax)
      set_window(0.0,Sxmax+SBDR_DIM,0.0,Sxmax+SBDR_DIM);
   else 
      set_window(0.0,Symax+SBDR_DIM,0.0,Symax+SBDR_DIM);
   create_temporary_segment();

   /* Draw the frame */
   move_abs_2(Sxmin,Symin);
   line_rel_2(Sxmax-Sxmin,0.0);
   line_rel_2(0.0,Symax-Symin);
   line_rel_2(Sxmin-Sxmax,0.0);
   line_rel_2(0.0,Symin-Symax);
   set_linewidth(0.50);

   /* Guide Label */
   if (sleeptime < 20) 
      strcpy(guide,"Press any key to quit");
   else 
      sprintf(guide,"Pausing %d seconds",sleeptime);
   set_charsize(10.0,10.0);
   Sxpos = (double)Sxmin;
   Sypos = Symax +  80.0;
   move_abs_2(Sxpos,Sypos);
   text(guide);

   /* Top Label */
   set_charsize(15.0,15.0);
   inquire_text_extent_2(toplabel,&cdx,&cdy);
   Sxpos = 0.5*(Sxmax+Sxmin)-0.5 * 1.1 * cdx;
   Sypos = Symax + cdy + 20.0;
   move_abs_2(Sxpos,Sypos);
   text(toplabel);

   /* Draw the tick marks and labels */
   if (!autorange) {
      plot_SUNticks();
   } else {
      plot_SUNAticks();
   }

   /* X-axis Label */
   set_charsize(10.0,10.0);
   inquire_text_extent_2(xlabel,&cdx,&cdy);
   Sxpos = 0.5*(Sxmax+Sxmin)-0.5 * 1.05 * cdx;
   Sypos = Symin - 40.0;
   move_abs_2(Sxpos,Sypos);
   text(xlabel);

   /* Y-axis Label */
   set_charup_2(-1.0,0.0);
   set_charpath_2(0.0,1.0);
   set_charsize(10.0,10.0);
   inquire_text_extent_2(ylabel,&cdx,&cdy);
   Sxpos = Sxmin - 40.0; 
   Sypos = 0.5*(Symax+Symin) - 0.5*1.05*cdy;
   move_abs_2(Sxpos,Sypos);
   text(ylabel);
   set_charup_2(0.0,1.0);
   set_charpath_2(1.0,0.0);

   /* reset */
   set_linewidth(0.20);

}

/* Draw the tick marks and labels */
plot_SUNticks()
{
   char            *sprintf();
   extern int      grid,xticks,yticks,xlog,ylog;
   extern double   xmin, xmax, ymin, ymax;

   double xdx,ydy,xintv,yintv;
   double Sxtmp,Sytmp;
   double vallbl;
   int    i;

   /* reset the number of ticks */
   if (xlog) {
      xticks = (int)(xmax - xmin - 1e-5) + 1;
      xmax   = xmin + xticks;
   }
   if (ylog) {
      yticks = (int)(ymax - ymin - 1e-5) + 1;
      ymax   = ymin + yticks;
   }

   /* Useful Data */
   xdx   = (xmax-xmin)/xticks;
   ydy   = (ymax-ymin)/yticks;
   xintv = (Sxmax-Sxmin)/xticks;
   yintv = (Symax-Symin)/yticks;

   /* Tick Marks on the X-axis */
   set_charsize(7.0,7.0);
   for (i=0; i<=xticks; i++) {
      Sxtmp = Sxmin + xintv*i;
      Sytmp = Symin;
      move_abs_2(Sxtmp,Sytmp);
      line_rel_2(0.0,10.0);

      vallbl = xmin + i*xdx;
      plot_SUNxticklabel(vallbl,Sxtmp);

      Sytmp = Symax;
      move_abs_2(Sxtmp,Sytmp);
      line_rel_2(0.0,-10.0);
   }

   /* Tick Marks on the Y-axis */
   for (i=0; i<=yticks; i++) {
      Sytmp = Symin + yintv*i;
      Sxtmp = Sxmin;
      move_abs_2(Sxtmp,Sytmp);
      line_rel_2(10.0,0.0);

      vallbl = ymin + i*ydy;
      plot_SUNyticklabel(vallbl,Sytmp);

      Sxtmp = Sxmax;
      move_abs_2(Sxtmp,Sytmp);
      line_rel_2(-10.0,0.0);
   }

   /* draw the grid */
   if (grid == ON) {
      for (i=0; i<=xticks; i++) {
         Sxtmp = Sxmin + xintv*i;
         Sytmp = Symin;
         set_linewidth(0.10);
         set_linestyle(DOTTED);
         move_abs_2(Sxtmp,Sytmp);
         line_rel_2(0.0,Symin-Symax);
         set_linewidth(0.50);
         set_linestyle(SOLID);
      }
      for (i=0; i<=yticks; i++) {
         Sytmp = Symin + yintv*i;
         Sxtmp = Sxmin;
         set_linewidth(0.10);
         set_linestyle(DOTTED);
         move_abs_2(Sxtmp,Sytmp);
         line_rel_2(Sxmin-Sxmax,0.0);
         set_linewidth(0.50);
         set_linestyle(SOLID);
      }
   }
}

/* Draw the tick marks and labels with automatic ranging */
plot_SUNAticks()
{
   char            *sprintf();
   double          log10();
   extern int      grid,xticks,yticks,xlog,ylog;
   extern double   xmin, xmax, ymin, ymax;

   double xmin2, xmax2, deltx, ymin2, ymax2, delty;
   double xtmp, ytmp, xt, yt, xfactor, yfactor;
   double Sxtmp, Sytmp;
   int    i, nticks;

   /*
    * useful data
    * X = Sxmin + (x-xmin)*(Sxmax-Sxmin)/(xmax-xmin)
    * Y = Symin + (y-ymin)*(Symax-Symin)/(ymax-ymin)
    */
   xfactor = (Sxmax - Sxmin)/(xmax-xmin);
   yfactor = (Symax - Symin)/(ymax-ymin);

   /* get rounded limits */
   get_autorange(xmin,xmax,ymin,ymax,
                 &xmin2,&xmax2,&deltx,&ymin2,&ymax2,&delty);

   /* Tick Marks on the X-axis */
   set_charsize(7.0,7.0);

   /* draw minor tick marks */
   xtmp = xmin2;
   while (xtmp < xmax) {
      for (i=1; i<10; i++) {
         xt = xtmp + i*0.1*deltx;
         if (xlog) xt = xtmp + log10((double)i);
         if ((xt > xmin) && (xt < xmax)) {
            Sxtmp = Sxmin + (xt-xmin)*xfactor;
            if (i==5) {
               move_abs_2(Sxtmp,Symin);
               line_rel_2(0.0, 7.0);
               move_abs_2(Sxtmp,Symax);
               line_rel_2(0.0,-7.0);
            } else {
               move_abs_2(Sxtmp,Symin);
               line_rel_2(0.0, 4.0);
               move_abs_2(Sxtmp,Symax);
               line_rel_2(0.0,-4.0);
            }
         }
      }
      xtmp += deltx;
   }

   /* major tickmarks and accompanying labels */
   xtmp = xmin2 + deltx;
   nticks = 0;
   while (xtmp < xmax-0.0001) {
      Sxtmp = Sxmin + (xtmp-xmin)*xfactor;

      move_abs_2(Sxtmp,Symin);
      line_rel_2(0.0, 10.0);

      plot_SUNxticklabel(xtmp,Sxtmp);

      move_abs_2(Sxtmp,Symax);
      line_rel_2(0.0,-10.0);

      xtmp += deltx;
      nticks ++;
   }

   /* put labels on the min and max points */
   if ((nticks==0) || ((xmin - xmin2) < 0.5*deltx)) {
      plot_SUNxticklabel(xmin,Sxmin);
   }

   /* put labels on the min and max points */
   if ((nticks==0) || ((xtmp - xmax) < 0.5*deltx)) {
      plot_SUNxticklabel(xmax,Sxmax);
   }

   /* Tick Marks on the Y-axis */

   /* draw minor tick marks */
   ytmp = ymin2;
   while (ytmp < ymax) {
      for (i=1; i<10; i++) {
         yt = ytmp + i*0.1*delty;
         if (ylog) yt = ytmp + log10((double)i);
         if ((yt > ymin) && (yt < ymax)) {
            Sytmp = Symin + (yt-ymin)*yfactor;
            if (i==5) {
               move_abs_2(Sxmin,Sytmp);
               line_rel_2( 7.0,0.0);
               move_abs_2(Sxmax,Sytmp);
               line_rel_2(-7.0,0.0);
            } else {
               move_abs_2(Sxmin,Sytmp);
               line_rel_2( 4.0,0.0);
               move_abs_2(Sxmax,Sytmp);
               line_rel_2(-4.0,0.0);
            }
         }
      }
      ytmp += delty;
   }

   /* major tickmarks and accompanying labels */
   ytmp = ymin2 + delty;
   nticks = 0;
   while (ytmp < ymax-0.0001) {
      Sytmp = Symin + (ytmp-ymin)*yfactor;

      move_abs_2(Sxmin,Sytmp);
      line_rel_2(10.0,0.0);

      plot_SUNyticklabel(ytmp,Sytmp);

      move_abs_2(Sxmax,Sytmp);
      line_rel_2(-10.0,0.0);

      ytmp += delty;
      nticks ++;
   }

   /* put labels on the min and max points */
   if ((nticks==0) || ((ymin - ymin2) < 0.5*delty)) {
      plot_SUNyticklabel(ymin,Symin);
   }

   /* put labels on the min and max points */
   if ((nticks==0) || ((ytmp - ymax) < 0.5*deltx)) {
      plot_SUNyticklabel(ymax,Symax);
   }

   /* draw the grid */
   if (grid == ON) {
      xtmp = xmin2 + deltx;
      while (xtmp < xmax-0.0001) {
         Sxtmp = Sxmin + (xtmp-xmin)*xfactor;
         Sytmp = Symin;
         set_linewidth(0.10);
         set_linestyle(DOTTED);
         move_abs_2(Sxtmp,Sytmp);
         line_rel_2(0.0,Symin-Symax);
         set_linewidth(0.50);
         set_linestyle(SOLID);
         xtmp += deltx;
      }
      ytmp = ymin2 + delty;
      while (ytmp < ymax-0.0001) {
         Sytmp = Symin + (ytmp-ymin)*yfactor;
         Sxtmp = Sxmin;
         set_linewidth(0.10);
         set_linestyle(DOTTED);
         move_abs_2(Sxtmp,Sytmp);
         line_rel_2(Sxmin-Sxmax,0.0);
         set_linewidth(0.50);
         set_linestyle(SOLID);
         ytmp += delty;
      }
   }
}

/* plot the tick label on the x-axis */
plot_SUNxticklabel(vallbl,Sxtmp)
double vallbl, Sxtmp;
{
   char   *sprintf();
   extern int     xlog;

   char   label[MAXCHAR];
   float  cdx, cdy;
   double Sxpos, Sypos;

   if (xlog) vallbl = pow(10.0,vallbl);
   sprintf(label,"%.3g",vallbl);
   inquire_text_extent_2(label,&cdx,&cdy);
   Sxpos = Sxtmp - 0.5*1.05*cdx;
   Sypos = Symin - 15.0;
   move_abs_2(Sxpos,Sypos);
   text(label);
}

/* plot the tick label on the y-axis */
plot_SUNyticklabel(vallbl,Sytmp)
double vallbl, Sytmp;
{
   char   *sprintf();
   extern int     xlog;

   char   label[MAXCHAR];
   float  cdx, cdy;
   double Sxpos, Sypos;

   /* vertical label orientation */
   set_charup_2(-1.0,0.0);
   set_charpath_2(0.0,1.0);

   if (ylog) vallbl = pow(10.0,vallbl);
   sprintf(label,"%.3g",vallbl);
   inquire_text_extent_2(label,&cdx,&cdy);
   Sxpos = Sxmin - 15.0;
   Sypos = Sytmp - 0.5*1.05*cdy;
   move_abs_2(Sxpos,Sypos);
   text(label);

   /* reset the label orientation */
   set_charup_2(0.0,1.0);
   set_charpath_2(1.0,0.0);
}

/* draw the curves */
plotsun()
{
   extern int     line,marker;
   extern int     linechange,markerchange;
   extern int     linetype[MAXTYPE], markertype[MAXTYPE], plottypes;
   extern double  xmin, ymin, xmax, ymax;

   segmptr S;
   nodeptr N;
   double  x,y,idx,jdy;
   int     kl, km, nplt=0;

   idx = (Sxmax-Sxmin)/(xmax-xmin);
   jdy = (Symax-Symin)/(ymax-ymin);

   kl = linetype[nplt];
   km = markertype[nplt];

   S = segmhead;
   for ( ; S!=NULL; S=S->next) {
      /* plot the points */
      if (line == ON) {
         linetypsun(kl,kl+1);
         N = S->head;
         x = (N->x-xmin)*idx + Sxmin;
         y = (N->y-ymin)*jdy + Symin;
         move_abs_2(x,y);
         N = N->next;
         for ( ; N!=NULL; N=N->next) {
            x = (N->x-xmin)*idx + Sxmin;
            y = (N->y-ymin)*jdy + Symin;
            line_abs_2(x,y);
         }
      }

      /* plot markers */
      if (marker==ON) {
         km = km % SUN_MKR;
         set_marker_symbol(km+32);
         N = S->head;
         for ( ; N!=NULL; N=N->next) {
            x = (N->x-xmin)*idx + Sxmin;
            y = (N->y-ymin)*jdy + Symin;
            marker_abs_2(x,y);
         }
      }
      /* increment the plot-type by increasing the array index */
      nplt++;
      if (nplt >= plottypes) nplt = nplt % plottypes;
      /* now look up the line and marker type */
      if (linechange   == ON) kl = linetype[nplt];
      if (markerchange == ON) km = markertype[nplt];
   }
}

/* choose the linetype */
linetypsun(style,color)
int style,color;
{
        int i;

        i = ( color * 30 + 10 ) % MAPSIZE ;
        set_line_index(i);

        style = style % SUN_LIN;
        switch (style) {
        case 0 : set_linestyle(DOTDASHED); break;
        case 1 : set_linestyle(SOLID); break;
        case 2 : set_linestyle(DOTTED); break;
        case 3 : set_linestyle(DASHED); break;
        default: set_linestyle(SOLID); break;
        }
}

/* end procedure */
endgrsun()
{
        extern int sleeptime;
        char string[10];
        int  len=10;

        if (sleeptime < 20) {
           /* wait for keyboard input */
           initialize_device(KEYBOARD,1);
           set_echo(KEYBOARD,1,1);
           await_keyboard(6000000000,1,string,&len);
           /* printf("Keyboard String = %s\n",string); */
        } else {
           /* wait for button - this doesn't work! */
           /*
           int    button_number;
           initialize_device(BUTTON,1);
           initialize_device(BUTTON,2);
           initialize_device(BUTTON,3);
           set_echo(BUTTON,1,1);
           set_echo(BUTTON,2,1);
           set_echo(BUTTON,3,1);
           await_any_button(1000000000,&button_number);
           printf("Button = %d\n",button_number);
           */
           sleep(((unsigned)sleeptime));
        }

        close_temporary_segment();
        close_surf();         /* Close the surface */
}

