#include <math.h>
#include <stdio.h>

#define SMALL 1.0e-5

/* round down or up the limits on the x-axis */
get_autorange(xmin,xmax,ymin,ymax,xmin2,xmax2,deltx,ymin2,ymax2,delty)
double  xmin,xmax,ymin,ymax,*xmin2,*xmax2,*deltx,*ymin2,*ymax2,*delty;
{
   double log10();
   double round_down(), round_up();
   double dpower;
   int    power;

   /* autorange on the x-axis */

   /* error-checking */
   if (fabs(xmax-xmin) < SMALL) {
      fprintf(stderr,"Error!  xmin = xmax = %10.5f!\n",xmin);
      *xmin2 = xmin;
      *xmax2 = xmax;
   } else {
      /* round down xmin and round up xmax */
      dpower = log10(xmax-xmin+SMALL);   
      power  = (dpower > -SMALL) ? (int)dpower : (int)dpower - 1;
      *xmin2 = round_down(xmin,power);
      *xmax2 = round_up(xmax,power);
      *deltx = pow(10.0,(double)power);
   }

   /* autorange on the y-axis */

   /* error-checking */
   if (fabs(ymax-ymin) < SMALL) {
      fprintf(stderr,"Error!  ymin = ymax = %10.5f!\n",ymin);
      *ymin2 = ymin;
      *ymax2 = ymax;
   } else {
      /* round down ymin and round up ymax */
      dpower = log10(ymax-ymin+SMALL);   
      power  = (dpower > -SMALL) ? (int)dpower : (int)dpower - 1;
      *ymin2 = round_down(ymin,power);
      *ymax2 = round_up(ymax,power);
      *delty = pow(10.0,(double)power);
   }
}

/* round down a number automatically to the nearest log scale */
double auto_round_down(x)
double x;
{
   double log10();
   double power();
   double x2, dpower;
   int    power;

   /* error-checking */
   if (fabs(x) < SMALL) {
      fprintf(stderr,"Error!  x = %10.5f!\n",x);
      exit(-1);
   } 

   /* round down xmin and round up xmax */
   dpower = log10(x+SMALL);   
   power  = (dpower > -SMALL) ? (int)dpower : (int)dpower - 1;
   x2     = pow(10.0,(double)power);
   return(x2);
}

/* round up a number automatically to the nearest log scale */
double auto_round_up(x)
double x;
{
   double log10();
   double power();
   double x2, dpower;
   int    power;

   /* error-checking */
   if (fabs(x) < SMALL) {
      fprintf(stderr,"Error!  x = %10.5f!\n",x);
      exit(-1);
   } 

   /* round down xmin and round up xmax */
   dpower = log10(x);   
   power  = (dpower > -SMALL) ? (int)dpower : (int)dpower - 1;
   if ((fabs(dpower - (double)power)) < SMALL)
      x2     = pow(10.0,(double)power);
   else 
      x2     = pow(10.0,(double)(power+1));
   return(x2);
}

/* round down a number */
double round_down(x,power)
double x;
int    power;
{
   double pow();
   double inc, xnew;
   
   inc  = pow(10.0,(double)power);
   if (x > -SMALL) 
      xnew = inc * ((int)(x/inc + SMALL));
   else 
      xnew = inc * ((int)(x/inc + SMALL) - 1);
   return(xnew);
}

/* round up a number */
double round_up(x,power)
double x;
int    power;
{
   double pow();
   double inc, xnew;
   
   inc  = pow(10.0,(double)power);
   if (x > SMALL) 
      xnew = inc * ((int)(x/inc - SMALL) + 1);
   else  
      xnew = inc * ((int)(x/inc - SMALL));
   return(xnew);
}
